#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
package ${package}.web.infra;

import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * このクラスは、JSF2Webアプリケーション内で
 * スタイルシート(CSS)内の相対パス指定を有効にするための{@code Filter}実装です。
 * 
 * <p>
 * JSF2では、CSSの外部参照を{@code h:outputStyesheet}タグで参照します。(ビューでの宣言)
 * このタグによって出力されるHTMLでは、CSSはJSFのサーブレット以下のURIでアクセスされます(出力されるHTML)。
 * このとき、CSS内でURL関数の相対パス指定を使ったリソース参照があると、
 * ブラウザはこのサーブレットからの相対パスへアクセスしてしまい(アクセスされるURI)、正しくリソースを参照できません。
 * このFilterは、このような不正なURIへのリソース参照のリクエストを、正しいURIへフォワードします。
 * </p>
 * 
 * <pre>
 * {@code 
 * ビューでCSSの使用を宣言：
 * <h:outputStylesheet library="css" name="style.css" />
 * 
 * 出力されるHTML：
 * <link type="text/css" rel="stylesheet" href="/contextRoot/javax.faces.resource/style.xhtml?ln=css"/>
 * 
 * CSS内のURL関数を用いた相対パス指定：
 * background:url(images/bg.png);
 * 
 * アクセスされるURI：
 * /javax.faces.resource/images/bg.png
 * 
 * このFilterのフォーワード先URI；
 * /contextRoot/javax.faces.resource/bg.xhtml?ln=images
 * }
 * </pre>
 * <p>
 * 
 * 
 * @author SIToolkit
 */
public class ResourceFilter implements Filter {

	private static final Logger LOG = LoggerFactory.getLogger(ResourceFilter.class);
	
	String resourceServletUri = "/javax.faces.resource/";

	String viewExt = ".xhtml";

	/**
	 * デフォルトコンストラクタです。何も処理を行いません。
	 */
	public ResourceFilter() {
	}

	/**
	 * 
	 * @param request
	 *            The servlet request we are processing
	 * @param response
	 *            The servlet response we are creating
	 * @param chain
	 *            The filter chain we are processing
	 * 
	 * @exception IOException
	 *                if an input/output error occurs
	 * @exception ServletException
	 *                if a servlet error occurs
	 */
	@Override
	public final void doFilter(final ServletRequest request,
			final ServletResponse response, final FilterChain chain)
			throws IOException, ServletException {

		HttpServletRequest req = HttpServletRequest.class.cast(request);
		HttpServletResponse res = HttpServletResponse.class.cast(response);

		String resourceUri = getResourceUri(req.getContextPath(),
				req.getRequestURI());

		if (resourceUri == null) {
			LOG.trace("フィルタ処理を行いません。resourceUri:{}", req.getRequestURI());
			chain.doFilter(request, response);
		} else {
			LOG.trace("フィルタ処理を行います。resourceUri:{}", resourceUri);
			req.getRequestDispatcher(resourceUri).forward(req, res);
		}
	}

	/**
	 * リソースへのURIを読み替えます。
	 * @param contextPath
	 *            Webアプリケーションのコンテキストパス
	 * @param resourceUri
	 *            リソースへのURI
	 * @return フォワード先URI
	 */
	String getResourceUri(String contextPath, String resourceUri) {
		if (resourceUri == null || resourceUri.endsWith(viewExt)) {
			return null;
		}

		int lastSlash = resourceUri.lastIndexOf("/");
		String resourceName = resourceUri.substring(lastSlash + 1);
		String resourcePath = resourceUri.replace(contextPath, "")
				.replace(resourceServletUri, "")
				.replace("/" + resourceName, "");

		return resourceServletUri + resourceName + viewExt + "?ln="
				+ resourcePath;
	}

	@Override
	public void init(FilterConfig filterConfig) throws ServletException {
	}

	@Override
	public void destroy() {
	}

}
